/*  demorse.h
 *
 *  Header file for demorse
 */

/*
 *  demorse: An application to decode Morse signals to text
 *
 *  Copyright (C) 2001  Neoklis Kyriazis
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details:
 *
 *  http://www.gnu.org/copyleft/gpl.txt
 */


#ifndef DEMORSE_H
#define DEMORSE_H 1

#include <signal.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <stdio.h>

/************ CUSTOMIZE THESE DEFINITIONS FOR YOUR NEEDS *************/

#define MIXER_DEV      "/dev/mixer" /* Your mixer device */
#define DSP_DEV          "/dev/dsp" /* Your dsp device   */
#define RECORD_SRC           "line" /* Signal recording source */
#define INPUT_LEV           "igain" /* Input level control device */
#define CAPTURE_LEV              50 /* Default level of line and igain */
#define NUM_CHANNELS              2 /* Stereo/Mono: 1=Mono, 2=Stereo   */
#define USE_CHANNEL           RIGHT /* Channel for capture LEFT|RIGHT  */
#define STEP_THRESHOLD			105 /* % diff between successive sig levels */
#define DET_THRESHOLD			 60 /* Mark/Space detector threshold */

/* Definitions for Morse code conversion to ascii */
#define NUMBER_OF_CHAR           55 /* Number of chars in ASCII table */

/* ASCII equivalents to Morse hex code. Last */
/* one (*) used for unrecognized characters. */
#define MORSE_ASCII_CHAR \
{ 'A','B','C','D','E','F','G','H','I','J','K','L','M', \
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z', \
  '1','2','3','4','5','6','7','8','9','0','.',',',':', \
  '?','\'','-','/','(','"',';','$','#','<','!','@',']',\
  '=','~',' ','*' }

/* Hex equivalents to Morse code chars above except (*).  */
/* Formed by starting with a 1 and following with a 0 for */
/* dash and 1 for dit e.g: A = ditdah = 101 = Hex 0x06.   */
#define MORSE_HEX_CODE \
{ 0x06,0x17,0x15,0x0b,0x03,0x1d,0x09,0x1f,0x07,0x18,0x0a,0x1b,0x04,\
  0x05,0x08,0x19,0x12,0x0d,0x0f,0x02,0x0e,0x1e,0x0c,0x16,0x14,0x13,\
  0x30,0x38,0x3c,0x3e,0x3f,0x2f,0x27,0x23,0x21,0x20,0x6a,0x4c,0x47,\
  0x73,0x61,0x5e,0x2d,0x52,0x6d,0x55,0xf6,0x35,0x7a,0x2a,0x37,0x29,\
  0x2e,0xff,0x01 }

/* Initial receiving speed in w.p.m */
#define INITIAL_WPM		20

/* Definitions for word wrap control */
#define LINE_LENGTH		80 /* Maximum line length if word wrap enabled */

/***************** END OF CUSTOMIZABLE SECTION ***********************/

/* General definitions for signal processing */
#define DSP_SPEED              8000 /* dsp sampling speed in samples/sec */
#define BUFFER_SIZE            1024 /* Size of buffer for signal samples */

/* Signal Detector parameter definitions */
#define AUDIO_FREQUENCY         500 /* Nominal detector frequency (Hz)  */
#define CYCLES_PER_FRAG           2 /* Cycles of signal/signal fragment */
#define MAX_UNIT_LEN             30 /* Max length of Morse unit, fragments */
#define MIN_UNIT_LEN             10 /* Max length of Morse unit, fragments */

/* Definitions of stages in Morse decoding process */
#define MARK_SIGNAL            0x0001 /* Count fragments of a mark element   */
#define ELEM_SPACE             0x0002 /* Count frag. of inter-element space  */
#define CHAR_SPACE             0x0004 /* Count fragments of inter-char space */
#define WAIT_WORD_SPACE        0x0008 /* Wait for an inter-word space        */
#define WORD_SPACE             0x0010 /* Count fragments of inter-word space */
#define WAIT_FOR_MARK          0x0020 /* Count fragments of no-signal space  */
#define WAIT_FOR_SPACE         0x0040 /* Wait for a space after a long dash  */
#define MARK_TONE              0x0080 /* Fragment is a mark (key down)       */

/* Definitions of flags for various actions */
#define ADAPT_SPEED            0x0100 /* Enable speed tracking */
#define WORD_WRAP              0x0200 /* Enable word wrapping  */
#define DSP_IO_ERROR           0x1000 /* Error reading from dsp */
#define SETUP_LEVEL			   0x2000 /* Enable audio level setup */
#define LEFT						0
#define RIGHT						1

/* demorse version string */
#define VERSION  "demorse 0.8"

/*-------------------------------------------------------------------*/

typedef struct
{
  char
	mix_dev[21], /* Mixer device name */
	dsp_dev[21], /* DSP device name   */
	rec_src[21], /* Recording/capture source */
	inp_lev[21]; /* Input level contol name  */

  int
	use_chn,  /* Channel in use: 0=left, 1=right */
	rec_lev,  /* Recording/Capture level */
	spd_wpm,  /* Morse speed WPM */
	wrd_wrp,  /* Maximum line length if word wrap enabled   */
	det_thr;  /* Signal threshold for Mark/Space separation */

} rc_data_t;

/*-------------------------------------------------------------------*/

/* Function prototypes produced by cproto */
/* decode.c */
char Get_Character(void);
void Adapt_Decoder(void);
char Hex_to_Ascii(int *hex_code);
/* demorse.c */
int main(int argc, char *argv[]);
void Usage(void);
void Abort_On_Error(int why);
int isFlagSet(int flag);
int isFlagClear(int flag);
void Set_Flag(int flag);
void Clear_Flag(int flag);
void Toggle_Flag(int flag);
/* detect.c */
int Get_Fragment(void);
/* sound.c */
void Setup_Sound_Card(void);
int Get_Signal_Sample(void);
void Setup_Signal_Input(void);

/*-------------------------------------------------------------------*/

#endif /* End of demorse.h */


